#include <iostream>
#include <sys/stat.h>
#include "../shared/classes.cpp"
#include "../shared/alg_file.hpp"
#include "../algorithm/algorithm.hpp"

// #define _TIMED_
#ifdef _TIMED_
	#include <time.h>
#endif

using namespace std;

int main()
{
	int n, i = 4, j, precision;
    struct stat sb;
	string filename_qml, filename_equiv;
	ofstream txt_file;
	mpz_class x;
	leaf mu_p(1,1,1,1), mu_q, mu_1o7(1,7,2,7), EOFL;
	#ifdef _TIMED_
		time_t start, now1, now2;
	#endif

	cout << "This program calculates mu_p and mu_q for "
			<< "s_{3/7}\\mate s_p \\equiv s_{1/7}\\mate s_q";

	// get n;
	cout << "\n\nWhat is the highest period of p you wish to consider?: ";
	cin >> n;

	#ifdef _TIMED_
		// Set the starting time
		time( &start );
	#endif
	// This is the main loop. It cycles through all periods less than or equal
	// to the maximum required period given by the user above.
	for( i = 4; i <= n; ++i ){
		// set filenames
	    x = i;
	    filename_qml = "qml_p";
	    filename_equiv = "equiv_ml_p";
	    if( x < 10 ){
	        filename_qml = filename_qml + '0';
	        filename_equiv = filename_equiv + '0';
		}
	    filename_qml = filename_qml + x.get_str() + ".txt";
	    filename_equiv = filename_equiv + x.get_str() + ".txt";

		// check that the qml file IS present and the equiv_ml file ISN'T
	    if( stat( filename_qml.c_str(), &sb ) || !stat( filename_equiv.c_str(), &sb ) ){
	        cout << filename_qml << " not present or " << filename_equiv << " exists. Omitting period " << i << endl;
	    }else{

			#ifdef _TIMED_
				// Get the current time and print the time since starting in seconds
				time( &now1 );
				cout<<'('<<now1-start<<"s): Calculating period "<<i<<" equivalences..." << endl;
			#else
				cout << "Calculating period " << i << " equivalences." << endl;
			#endif

			// open the txt file for writing the equivalences too and ready
			// `filename' for opening the relavent QML files.
			txt_file.open( filename_equiv.c_str() );

			j = 0;
			// get first leaf of period `i' from the QML;
			mu_p = get_leaf( filename_qml, j++, 1, i );
			// This next while loop cycles through all minor leaves of period
			// `i'
			while( mu_p != EOFL ){

				// if leaf \in (1/7,2/7)
				if( rat_in( mu_p.get_a(), mu_1o7 ) ){
					// find equiv. mating
					precision = 3;
					mu_q = algorithm( mu_p, false, precision );

					// short unsigned k = 2;
					while( mu_q.get_a().get_denom() == 0 && (mu_q.get_a().get_num() == 4 || mu_q.get_a().get_num() == 5 ) ){
						cout<<"\tStall on "<< mu_p <<". Upping precision ( to " << precision + 1 << " )." << endl;
						mu_q = algorithm( mu_p, false, ++precision );
					}

					// Handle any errors from algorithm()
					if(mu_q.get_a().get_denom() == 0){
						switch( mu_q.get_a().get_num().get_ui() ){
							case 1:
								cout<< "Trivial mu_p (" << mu_p << ").\n";
								break;
							case 2:
								cout<< "Not a valid minor leaf (" <<mu_p<< ").\n";
								break;
							case 3:
								cout<<"Forward image of minor leaf self intersects"
									<<" ("<<mu_p<<").\n";
								break;
							case 4:
								cout<<"Uninterpretable vague word ("<<mu_p<<").\n";
								break;
							case 5:
								cout<<"No mu_q could be found in the range given by"
									<<" this algorithm ("<<mu_p<<").\n";
								break;
							case 6:
								cout<<"QML file of period "<<i<<" does not exist."
									<<" You can genterate this with `qml.out'\n";
								break;
							case 7:
								cout<<"The vague and full words differ in length."
									<<" ("<<mu_p<<").\n";
								break;
							default:
								cout<<"Error on "<<mu_p<<".\n";
						}
						//return 1;
					}else{
						// write mu_p and mu_q to text file
						txt_file << mu_p << '	' << mu_q << endl;
					}
				}
				// get next leaf;
				mu_p = get_leaf( filename_qml, j++, 1, i );
			}

			txt_file.close();

			#ifdef _TIMED_
				// Get the current time and print the time since starting and time
				// since starting THIS PERIOD
				time( &now2 );
				cout << "(" << now2 - start << "s): done. Task took " << now2 - now1 << " seconds." << endl;
			#endif
		}
	}

	return 0;
}
