#include <iostream>
#include <fstream>
#include <Board.h>
#include <sys/stat.h>
#include "../shared/calc_leaf.hpp"
#include "../shared/classes.cpp"

using namespace std;
using namespace LibBoard;

void get_equiv_leaves( string filename, int n, leaf &a, leaf &b )
{
	int i = 0;
	rat r;

	// open file
	ifstream file( filename.c_str() );

	// skip n lines
	for( ; i < n; ++i )
		file.ignore( 512, '\n' );

	// get a and b
	if( file.eof() == true ){
		a = b = leaf();
	} else {
		file >> r;
		a.set_a( r );
		file.ignore(1);
		file >> r;
		a.set_b( r );
		file.ignore(1);
		file >> r;
		b.set_a( r );
		file.ignore(1);
		file >> r;
		b.set_b( r );
		file.ignore(1);
	}
}


int main()
{
	int n, i = 4, j, precision;
    struct stat sb;
	string filename_equiv;
	mpz_class x;
	leaf mu_p(1,1,1,1), mu_q, mu_1o7(1,7,2,7), EOFL;
	Board b;

	cout << "This program draws mu_p and mu_q for "
			<< "s_{3/7}\\mate s_p \\equiv s_{1/7}\\mate s_q";

	// get n;
	cout << "\n\nWhat is the highest period of p you wish to consider?: ";
	cin >> n;

	// draw circle
    b.setLineWidth( 1 ).setPenColorRGBi( 0, 0, 0 ).drawCircle( 300, 300, 250, 0 );

	// This is the main loop. It cycles through all periods less than or equal
	// to the maximum required period given by the user above.
	for( i = 4; i <= n; ++i ){
		// set filenames
	    x = i;
	    filename_equiv = "equiv_ml_p";
	    if( x < 10 ){
	        filename_equiv = filename_equiv + '0';
		}
	    filename_equiv = filename_equiv + x.get_str() + ".txt";

	    if( stat( filename_equiv.c_str(), &sb ) ){
	        cout << filename_equiv << " not present. Omitting period " << i << endl;
	    }else{

			Group g;

			cout << "Drawing period " << i << " equivalences." << endl;

			j = 0;
			// get first leaf of period `i' from the QML;
			get_equiv_leaves( filename_equiv, j++, mu_p, mu_q );
			// This next while loop cycles through all minor leaves of period
			// `i'
			while( mu_p != EOFL ){

				// draw mu_p and mu_q and write to text file
				g << calc_leaf( mu_q.get_a(), mu_q.get_b(), 300, 300, 250, n );
				g << calc_leaf( mu_p.get_a(), mu_p.get_b(), 300, 300, 250, n, Color::Blue );

				// get next leaf;
				get_equiv_leaves( filename_equiv, j++, mu_p, mu_q );
			}

			b << g;

		}
	}

	x = n;
	filename_equiv = "equiv_ml_p";
	if( n < 10 )
		filename_equiv = filename_equiv + '0';
	filename_equiv = filename_equiv + x.get_str();

	#ifndef EPS
		filename_equiv = filename_equiv + ".svg";
		b.saveSVG( filename_equiv.c_str() );
	#else
		filename_equiv = filename_equiv + ".eps";
		b.saveEPS( filename_equiv.c_str() );
	#endif

	return 0;
}
