#if !defined( ALGFILE )
	#define ALGFILE

// small programs for the algorithm project
#include <fstream>
#include <string>
#include "leaf_helpers.hpp"
using namespace std;

int dppi( mpz_class i );

//------------------------------------------------------------------------------
// get_leaf( FILE* file, short ) - gets the next leaf from a Lam leaf file
//------------------------------------------------------------------------------

// returns EOFL if EOF
leaf get_leaf( const string & filename, int leaf_no, short type, int period = 0 )
{
	//unsigned int charLong = sizeof( long );
	mpz_class i = 0, j = 0;
	char c, d;
	leaf l;
	rat r1( 0, 0 ), r2( 0, 0 );

	fstream file( filename.c_str(), ios::in );
	leaf_no--;
	for( ; leaf_no > 0; leaf_no-- )
		file.ignore( 1024, '\n' );

	switch( type ){
		case 1:	// Get a leaf from a QML file
			if( file.is_open() && !file.eof() ){
				// Get numerators
				file>>skipws>>i>>j;

				// Get denominator
				if( i != 0 || j != 0 ){
					r1.set_num( i );
					r2.set_num( j );
					mpz_ui_pow_ui( i.get_mpz_t(), 2, period );	// i = 2 ^ period
					i--;
					r1.set_denom( i );
					r2.set_denom( i );
				}

				// Set leaf
				l.set_a( r1 );
				l.set_b( r2 );
			}
			break;

		case 2:	// Get a leaf from a lamination file
			if( file.is_open() && !file.eof() ){
				// Get first endpoint
				file>>skipws>>i>>c>>j;
				r1.set_num( i );
				r1.set_denom( j );

				// Get second endpoint
				file>>skipws>>i>>c>>j;
				r2.set_num( i );
				r2.set_denom( j );

				// Get label
				file>>skipws>>c;

				// Set leaf
				l.set_a( r1 );
				l.set_b( r2 );
			}
			break;

		case 3:	// Get a leaf from a Lt ( less than ) file
			if( file.is_open() && !file.eof() ){
				// Get denominator
				file>>skipws>>i;
				r1.set_denom( i );
				r2.set_denom( i );

				// Get numerators
				file>>skipws>>i>>j>>c;
				r1.set_num( i );
				r2.set_num( j );

				// Set leaf
				l.set_a( r1 );
				l.set_b( r2 );
			}
			break;

		case 4:	// Get a leaf from a Mu file
			if( file.is_open() && !file.eof() ){

				// Get numerators
				file>>skipws>>i>>j>>c;
				r1.set_num( i );
				r2.set_num( j );

				// Get denominator
				if( i != 0 || j != 0 ){
					mpz_ui_pow_ui( i.get_mpz_t(), 2, period );	// i = 2 ^ period
					i--;
					r1.set_denom( i );
					r2.set_denom( i );
				}

				// Set leaf
				l.set_a( r1 );
				l.set_b( r2 );
			}
			break;

		case 5:	// Get a leaf from a Lt ( less than ) file
			if( file.is_open() && !file.eof() ){
				// Get first endpoint
				file>>skipws>>i;
				file.ignore( 1 );
				file>>skipws>>j;
				r1.set_num( i );
				r1.set_denom( j );

				// hyphen
				file.ignore( 1 );

				// Get second endpoint
				file>>skipws>>i;
				file.ignore( 1 );
				file>>skipws>>j;
				r2.set_num( i );
				r2.set_denom( j );

				// Set leaf
				l.set_a( r1 );
				l.set_b( r2 );
			}
			break;

		default:
			break;
		}

		return l;
}

//------------------------------------------------------------------------------
// get_rat( char[20] file, short n ) - gets leaf 'n' rat from a file
//------------------------------------------------------------------------------

// returns EOFL if EOF
rat get_rat( char filename[20], int rat_no )
{
	//unsigned int charLong = sizeof( long );
	mpz_class i = 0, j = 0;
	char c;
	rat r1( 0, 0 );

	ifstream file( filename );
	rat_no--;
	for( ;rat_no>0;rat_no-- )
		file.ignore( 1024, '\n' );

	if( file.is_open() && !file.eof() ){
		// Get num and denom
		file>>skipws>>i;
		file.ignore( 1 );
		file>>skipws>>j;
		r1.set_num( i );
		r1.set_denom( j );
	}

	return r1;
}
//------------------------------------------------------------------------------
// put_leaf( leaf, char[20] ) - puts the next leaf into a Lam leaf file
//------------------------------------------------------------------------------

void put_leaf( const rat & a, const rat & b, const string & filename, short type )
{
	// Open file
	ofstream file( filename.c_str(), ios::out|ios::app );

	switch( type ){
		case 1:	// Put a leaf into a QML file
			// Write to file
			if( file.is_open() ){
				// Put numerators
				file << a.get_num() << ' ' << b.get_num() << endl;
			}
			break;

		case 2:	// Put a leaf into a lamination file
			// Write to file
			if( file.is_open() ){
				// Put leaf
				file << a << ' ';
				file << b << endl;
			}
			break;

		case 3:	// Put a leaf into a Lt ( less than ) file
			// Write to file
			if( file.is_open() ){
				file << a.get_denom() << ' ' << a.get_num() << ' ' << b.get_num() << endl;
			}
			break;

		case 4: // Put leaf into a Mu file
			// Write to file
			if( file.is_open() ){
				// Put numerators
				file << a.get_num() << ' ' << b.get_num() << endl;
			}
			break;

		default:
			break;
	}

	// Close file
	file.close();
}

void put_leaf( const leaf & l, const string & filename, short type)
{
	put_leaf( l.get_a(), l.get_b(), filename, type );
}

//------------------------------------------------------------------------------
// put_tri( tri T, char filename[20] ) - writes triangle to file
//------------------------------------------------------------------------------

void put_tri( tri T, char filename[20] )
{
	// Open file
	ofstream file( filename, ios::out|ios::app );

	// Write to file
	if( file.is_open() ){
		// Put numerators
		file<<T.get_a()<<' '<<T.get_a()<<' '<<T.get_a()<<endl;
	}

	// Close file
	file.close();
}

//------------------------------------------------------------------------------
// get_tri( char filename, int tri_no ) - gets the next triangle from a Lam.tri file
//------------------------------------------------------------------------------


tri get_tri( char filename[20], int tri_no = 0 )
{
	unsigned long i = 0, j = 0;
	char c = '/';
	tri t;

	fstream file( filename, ios::in );
	tri_no--;
	for( ; tri_no>0; tri_no-- )
		file.ignore( 1024, '\n' );

	if( file.is_open() && !file.eof() ){

		// Get A
		file>>skipws>>i>>c>>j;
		t.set_a( i, j );

		// Get B
		file>>i>>c>>j;
		t.set_b( i, j );

		// Get C
		file>>i>>c>>j;
		t.set_c( i, j );

	}

	return t;
}

//------------------------------------------------------------------------------
// void QMLname( int p )
//------------------------------------------------------------------------------

void QMLname( int p, char* filename )
{
	int i = 4, j, k, m, n;

	filename[0] = 'Q';
	filename[1] = 'M';
	filename[2] = 'L';
	filename[3] = '_';

	// put the number into the char array
	m = dppi( p );	// no. d.p.s in p
	for( n = ( m - 1 ); n >= 0; n-- ){
		j=p;
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}

	filename[i++] = '.';
	filename[i++] = 'l';
	filename[i++] = 'e';
	filename[i++] = 'a';
	filename[i++] = 'v';
	filename[i++] = 'e';
	filename[i++] = 's';
	filename[i++] = '\0';
}

//------------------------------------------------------------------------------
// void Lam_name( rat r0, char* filename )
//------------------------------------------------------------------------------

void Lam_name( rat r0, char* filename )
{
	int i = 4, j;	//, k, m, n;
	string s_number;

	filename[0] = 'L';
	filename[1] = 'a';
	filename[2] = 'm';
	filename[3] = '_';

	// put the numerator into the char array
	s_number = r0.get_num().get_str();

	for( j = 0; j < s_number.length(); ++j ){
		filename[ i++ ] = s_number.at(j);
	}
	/*m = dppi( r0.get_num() );	// no. d.p.s in p
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = r0.get_num();
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}*/

	filename[ i++ ] = '_';

	// put the denominator into the char array
	s_number = r0.get_denom().get_str();

	for( j = 0; j < s_number.length(); ++j ){
		filename[ i++ ] = s_number.at(j);
	}
	/*m = dppi( r0.get_denom() );	// no. d.p.s in p
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = r0.get_denom();
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}*/

	filename[i++] = '.';
	filename[i++] = 'l';
	filename[i++] = 'e';
	filename[i++] = 'a';
	filename[i++] = 'v';
	filename[i++] = 'e';
	filename[i++] = 's';
	filename[i++] = '\0';
}

//------------------------------------------------------------------------------
// void MUname( int p )
//------------------------------------------------------------------------------

void MUname( rat r, int p, char* filename )
{
	int i = 3, j, k, n, m;
	string s_number;

	filename[0] = 'M';
	filename[1] = 'u';
	filename[2] = '_';

	// put the numerator into the char array
	s_number = r.get_num().get_str();

	for( j = 0; j < s_number.length(); ++j ){
		filename[ i++ ] = s_number.at(j);
	}
	/*m = dppi( num );	// no. d.p.s in num
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = num;
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}*/

	filename[ i++ ] = '_';

	// put the denominator into the char array
	s_number = r.get_denom().get_str();

	for( j = 0; j < s_number.length(); ++j ){
		filename[ i++ ] = s_number.at(j);
	}
	/*m = dppi( d );	// no. d.p.s in d
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = d;
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}*/

	filename[ i++ ] = '_';

	// put the period into the char array
	m = dppi( p );	// no. d.p.s in p
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = p;
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[ i++ ] = '0' + j;
	}

	filename[i++] = '.';
	filename[i++] = 'l';
	filename[i++] = 'e';
	filename[i++] = 'a';
	filename[i++] = 'v';
	filename[i++] = 'e';
	filename[i++] = 's';
	filename[i++] = '\0';
}

//------------------------------------------------------------------------------
// Tri_name - stores triangles of L_{1/7}
//------------------------------------------------------------------------------

void Tri_name( char* filename )
{
	filename[0] = 'L';
	filename[1] = 'a';
	filename[2] = 'm';
	filename[3] = '.';
	filename[4] = 't';
	filename[5] = 'r';
	filename[6] = 'i';
	filename[7] = '\0';
}

//------------------------------------------------------------------------------
// void txt_name( int p, char* filename )
//------------------------------------------------------------------------------

void txt_name( int p, char* filename )
{
	int i = 10, j, k, m, n;

	filename[0] = 'e';
	filename[1] = 'q';
	filename[2] = 'u';
	filename[3] = 'i';
	filename[4] = 'v';
	filename[5] = '_';
	filename[6] = 'm';
	filename[7] = 'l';
	filename[8] = '_';
	filename[9] = 'p';

	// put the number into the char array
	m = dppi( p );	// no. d.p.s in p
	for( n = ( m - 1 ); n >= 0; n-- ){
		j = p;
		for( k = 0; k < n; ++k )
			j /= 10;
		j %= 10;	// just the last digit
		filename[i++] = '0' + j;
	}

	filename[i++] = '.';
	filename[i++] = 't';
	filename[i++] = 'x';
	filename[i++] = 't';
	filename[i++] = '\0';
}

//------------------------------------------------------------------------------
// int dppi( mpz_class i ) - Returns the number of decimal places from i
//------------------------------------------------------------------------------

int dppi( mpz_class i ){

	mpz_class n = 1;
	int m = 0;
	while( n <= i ){
		n *= 10;
		++m;
	}

	return m;
}

#endif
