#ifdef CALC_DEBUG
	#include <iostream>
#endif
#include "classes.cpp"
#include <Board.h>
#include "leaf_helpers.hpp"

using namespace LibBoard;

const double ARC=0;	// PI/2 - arc length of arc on smaller circles based
					// at A and B.

Group calc_leaf( rat alpha, rat beta, int centreX = 300, int centreY = 300,int radius = 250, short int depth = -1, Color colour = Color::Red, float linewidth = 0.5, bool label_endpoints = false ){

#ifdef CALC_DEBUG
	std::cout << "Calc'ing leaf: " << alpha << " - " << beta << std::endl;
#endif

    int i, j;
    double ax, ay, bx, by, dx, dy, cx, cy;
    double T=1, AX, AY, BX, BY;
    rat distance( alpha ), half( 1, 2 );
	Group g;

    // make sure alpha has a larger angle than beta
    if( alpha < beta ){   // if alapha is smaller than beta
        alpha = beta; // swap them around
        beta = distance;
    }

	dx = 2 * M_PI * alpha.get_num().get_d() / alpha.get_denom().get_d();
	dy = 2 * M_PI * beta.get_num().get_d() / beta.get_denom().get_d();

    ax = radius * cos( dx ) + centreX;
    ay = radius * sin( dx ) + centreY;
    bx = radius * cos( dy ) + centreX;
    by = radius * sin( dy ) + centreY;

    distance = dist( alpha, beta );
    if( distance.get_denom() != 2 && distance.get_num() != 0 ){ //alpha, beta not diametrically opposite or equal
        // calculate centre of arc
        AX = ( ax - centreX );
        AY = ( ay - centreY );
        BX = ( bx - centreX );
        BY = ( by - centreY );
        T = ( ( AX * AX + AY * AY ) - AX * BX - AY * BY ) / ( AX * BY - AY * BX );
        cx = bx + ( T * BY );
        cy = by - ( T * BX );

        // calculate radius, T, of arc
        T = sqrt( ( cx - ax ) * ( cx - ax ) + ( cy - ay ) * ( cy - ay ) );

		// if the arc spans 0 then note to write the arc as anti-clockwise
		if( ( beta - alpha ) < half ){
			g << Arc( cx, cy, bx, by, ax, ay, T, T, colour, Color::None, linewidth, depth ); // Check this
		} else {
			g << Arc( cx, cy, ax, ay, bx, by, T, T, colour, Color::None, linewidth, depth ); // Check this
		}
    }else{
		g << Line ( ax, ay, bx, by, colour, linewidth, Shape::ButtCap, Shape::MiterJoin, depth );
    }

	if( label_endpoints ){

   		// Label endpoints
	    ax = ( ( radius + ( radius / 12 ) ) * cos( dx ) + centreX );
	    ay = ( ( radius + ( radius / 12 ) ) * sin( dx ) + centreY );
	    bx = ( ( radius + ( radius / 12 ) ) * cos( dy ) + centreX );
	    by = ( ( radius + ( radius / 12 ) ) * sin( dy ) + centreY );

		string s, t;
	    // cancel factors in alpha
	    alpha.simplify();
		s = alpha.get_num().get_str() + '/' + alpha.get_denom().get_str();

	    // cancel factors in beta
	    beta.simplify();
		t = beta.get_num().get_str() + '/' + beta.get_denom().get_str();

		#ifdef TEXT_DEBUG
			cout << "For A: x = " << ax << ", y = " << ay << ", alpha = " << alpha << ", s = " << s << endl;
			cout << "For B: x = " << bx << ", y = " << by << ", beta = " << beta << ", t = " << t << endl;
		#endif

		// Text(double x, double y, const std::string &text, const Fonts::Font font, float size, Color color=Color::Black, int depth=-1)
		g << Text( ax, ay, s, Fonts::Courier, radius / 25, Color::Black, depth );
		g << Text( bx, by, t, Fonts::Courier, radius / 25, Color::Black, depth );

		// set ax, ay to be biggest
		if ( ay < by ) {
			cy = ay;
			ay = by;
			by = cy;
		}
		ay = ay + ( radius / 25 );
		by = by + ( radius / 25 );
		ax = ax + ( s.size() * ( radius / 25 ));
		bx = bx + ( t.size() * ( radius / 25 ));
		if ( ax < bx ) {
			cy = ay;
			ax = bx;
			by = cy;
		}

		// add rectangle to fix BB
		Rectangle R( bx, ay, ax - bx, ay - by, Color::None, Color::None, 0 );
		if ( ( R.y() > g.boundingBox().top ) || ( ( R.x() + R.width() ) > ( g.boundingBox().left + g.boundingBox().width ) ) )
			g << R;
	}

	return g;
}


//---------------------------------------------------------------------
// Group calc_leaf( rat A, rat B, int centre_x, int centre_y, int radius )
//	- draws a leaf on the exterior of the circle S with radius radius and
//	centre ( centre_x, centre_y ). It does this by drawing two smaller
//	circles which are perp to S at A and B, of radius rS (see formula,
//	it's pretty arbitry). An arc of rotation length PI/2-ARC, based at A or
//	B, is taken, together with an arc which joins the free ends of these
//	arcs smoothly.
//---------------------------------------------------------------------

Group calc_leaf_outside( rat A, rat B, int centre_x = 300, int centre_y = 300, int radius = 250, short int depth = -1, Color colour = Color::Red, float linewidth = 0.5, bool label_endpoints = false ){

	double theta, arc;
	Point temp_point, temp_vec;
	double rS, rL;	// small and large radii
	Point pA, pB; // x,y coords for A and B
	Point cA, cB; // centres of circles tangent to A
				// and B
	Point cM;	// centre of large circle
	Point eA, eB;	// endpoints of the arcs used from
					// the small circles tangent to A and B
	rat half( 1, 2 );
	rat offset; // holds how much A and B have been rotated out of
				// position. Undo at end.
	Point cCirc( centre_x, centre_y );	// holds the centre of the
										// circle to be drawn on.

	// invert rotations (put into inside labelling)
	A.set_num( A.get_denom() - A.get_num() );
	B.set_num( B.get_denom() - B.get_num() );

	// calculate offset and rotate smaller, A, to 0.
	offset.set_num( A.get_denom() - A.get_num() );
	offset.set_denom( A.get_denom() );
	B = B - A;
	if( B.get_num() < 0 ){
		B.set_num( B.get_num() + B.get_denom() );
	}
	A.set_num( 0 );
	if( B > half ){	// B is in the lower semicircle
		// rotate B, ccw, to 0 and A accordingly
		// add B to A
		offset = offset - B;
		if( offset.get_num() < 0 )
			offset.set_num( offset.get_denom() );
		B.set_num( B.get_denom() - B.get_num() );
	}

	// claculate arc: used to make small leaves closer to circles than
	// larger leaves.
	theta = B.get_num().get_d() / B.get_denom().get_d();
	arc = M_PI * ( 0.5 - theta ) / 2;

	// calc radii ( magic made up formula )
	rS = radius * 3 * theta / 2;

	// set centres
	cA.x = centre_x + radius;
	cA.y = centre_y + rS;
	// create centre for B at rotation 0 and rotate into position
	cB.x = centre_x + radius;
	cB.y = centre_y - rS;
	theta = 2 * M_PI * B.get_num().get_d() / B.get_denom().get_d();
	cB.rotate( theta, cCirc );

	// find endpoints of arc with rotation length PI/3 from A/B resp.
	eA.x = cA.x + cos( arc ) * rS;
	eA.y = cA.y - sin( arc ) * rS;
	eB.x = eA.x;
	eB.y = 2 * centre_y - eA.y;
	eB.rotate( theta, cCirc );

	// find centre of circle on which long arc joining two smaller arcs
	// lie. We do this by using the appropriate tangent of each small
	// circle. We rotate both around centre cA and shift down by cA_y so
	// that tangent at A's circle is perp to x axis, compute centre, and
	// then rotate back.
	cM.y = cA.y;
	// if following is true then the two lines used to find cM are coincident.
	if( ( M_PI * 2 * B.get_num().get_d() / B.get_denom().get_d() ) + ( 2 * arc ) == M_PI ){
		cM.x = cCirc.rotated( arc ).x;
		rL = cA.x + rS - cM.x;
	} else {
		temp_point.x = cB.x;
		temp_point.y = cB.y;
		temp_point.rotate( arc, cA );
		// calc unrotated vector
		temp_vec = eB - cB;
		// rotate around origin as end of vector
		temp_vec.rotate( arc );

		cM.x = temp_point.x + temp_vec.x * ( ( cA.y - temp_point.y ) / temp_vec.y );
		rL = cA.x + rS - cM.x;
		cM.rotate( -arc, cA );
	}

	#ifdef DRAW_DEBUG
		cout << "\nS^1 centre = ( " << centre_x << ", " << centre_y << " ), radius = " << radius << endl;
		cout << "A = ( " << pA.x << ", " << pA.y << " )" << endl;
		cout << "B = ( " << pB.x << ", " << pB.y << " )" << endl;
		cout << "circle at A centre = ( " << cA.x << ", " << cA.y << " ), radius = " << rS << endl;
		cout << "circle at B centre = ( " << cB.x << ", " << cB.y << " ), radius = " << rS << endl;
		cout << "end of arc from A = ( " << eA.x << ", " << eA.y << " )" << endl;
		cout << "end of arc from B = ( " << eB.x << ", " << eB.y << " )" << endl;
		cout << "M centre = ( " << cM.x << ", " << cM.y << " ), radius = " << rL << endl;
	#endif

	// undo offset
	A = A - offset;
	B = B - offset;
	if( A.get_num() < 0 )
		A.set_num( A.get_num() + A.get_denom() );
	if( B.get_num() < 0 )
		B.set_num( B.get_num() + B.get_denom() );

	// rotate back to original position according to offset
	theta = - 2 * M_PI * offset.get_num().get_d() / offset.get_denom().get_d();
	cA.rotate( theta, cCirc );
	cB.rotate( theta, cCirc );
	cM.rotate( theta, cCirc );
	eA.rotate( theta, cCirc );
	eB.rotate( theta, cCirc );

	// Calc x,y coords for A and B
	pA.y = 2 * M_PI * A.get_num().get_d() / A.get_denom().get_d();	// temperary
	pA.x = cos( pA.y ) * radius + centre_x;
	pA.y = sin( pA.y ) * radius + centre_y;
	pB.y = 2 * M_PI * B.get_num().get_d() / B.get_denom().get_d();	// temperary
	pB.x = cos( pB.y ) * radius + centre_x;
	pB.y = sin( pB.y ) * radius + centre_y;

	// draw arcs
	Group g( 1 );
	Arc aA( cA.x, cA.y, pA.x, pA.y, eA.x, eA.y, rS, rS, colour, Color::None, linewidth );
	Arc aB( cB.x, cB.y, eB.x, eB.y, pB.x, pB.y, rS, rS, colour, Color::None, linewidth );
	Arc aM( cM.x, cM.y, eA.x, eA.y, eB.x, eB.y, rL, rL, colour, Color::None, linewidth );
	g << aA << aB << aM;

	if( label_endpoints ){

		string s, t;
		double rot_a = 2 * M_PI * A.get_num().get_d() / A.get_denom().get_d();
		double rot_b = 2 * M_PI * B.get_num().get_d() / B.get_denom().get_d();

   		// Label endpoints
	    double ax = ( ( radius - ( radius / 12 ) ) * cos( rot_a ) + centre_x );
	    double ay = ( ( radius - ( radius / 12 ) ) * sin( rot_a ) + centre_y );
	    double bx = ( ( radius - ( radius / 12 ) ) * cos( rot_b ) + centre_x );
	    double by = ( ( radius - ( radius / 12 ) ) * sin( rot_b ) + centre_y );

	    // cancel factors in alpha
		A.set_num( A.get_denom() - A.get_num() );
	    A.simplify();
		s = A.get_num().get_str() + '/' + A.get_denom().get_str();

	    // cancel factors in beta
		B.set_num( B.get_denom() - B.get_num() );
	    B.simplify();
		t = B.get_num().get_str() + '/' + B.get_denom().get_str();

		// Text(double x, double y, const std::string &text, const Fonts::Font font, float size, Color color=Color::Black, int depth=-1)
		g << Text( ax, ay, s, Fonts::Courier, radius / 25, Color::Black, depth );
		g << Text( bx, by, t, Fonts::Courier, radius / 25, Color::Black, depth );
	}

	return g;
}
