#if !defined(CLASSES)
   #define CLASSES

#include <iostream>
#include <gmpxx.h>
#include "classes.hpp"

using namespace std;

ostream& operator<<(ostream& os, const rat& r0);
ostream& operator<<(ostream& os, const leaf& l);
ostream& operator<<(ostream& os, const tri& t);

 istream& operator>>(istream& os, rat& r0);
// istream& operator>>(istream& os, leaf& l);

//------------------------------------------------------------------------------
// class rat member functions (rational number)
//------------------------------------------------------------------------------

rat::rat()
{
	m_mpNum = 0;
	m_mpDenom = 1;
}

rat::rat(mpz_class p, mpz_class q)
{
	m_mpNum = p;
	m_mpDenom = q;
}

void rat::simplify()
{
	mpz_class factor;
	mpz_gcd( factor.get_mpz_t(), m_mpDenom.get_mpz_t(), m_mpNum.get_mpz_t() );

	m_mpDenom = m_mpDenom/factor;
	m_mpNum = m_mpNum/factor;
}

bool rat::operator==(rat q) const
{
	if(m_mpDenom == q.get_denom()){	// denominators equal
		if(m_mpNum == q.get_num())
			return true;
		else
			return false;
	}else{
		if((m_mpDenom != 0) && (q.get_denom() != 0)){
			mpz_class cm, liPNum = 1, liQNum = 1;
			mpz_lcm( cm.get_mpz_t(), m_mpDenom.get_mpz_t(), q.get_denom().get_mpz_t() );

			liPNum = (cm/m_mpDenom) * m_mpNum;
			liQNum = (cm/q.get_denom()) * q.get_num();
			if(liPNum == liQNum)
				return true;
			else
				return false;
		}else
			return false;
	}
}

bool rat::operator!=(rat q) const
{
	return !(*this == q);

	/*if(m_mpDenom == q.get_denom()){	// denominators equal
		if(m_mpNum == q.get_num())
			return false;
		else
			return true;
	}else{
		if((m_mpDenom != 0) && (q.get_denom() != 0)){
			long cm = lcm(m_mpDenom, q.get_denom()), liPNum = 1, liQNum = 1;
			liPNum = (cm/m_mpDenom) * m_mpNum;
			liQNum = (cm/q.get_denom()) * q.get_num();
			if(liPNum == liQNum)
				return false;
			else
				return true;
		}else
			return true;
	}*/
}

rat rat::operator+(rat q) const
{
	rat p;

	mpz_class m,c;

	if(q.get_denom() != m_mpDenom){
    	mpz_lcm( m.get_mpz_t(), m_mpDenom.get_mpz_t(), q.get_denom().get_mpz_t() );

    	//normalise p
    	c=m/m_mpDenom;
    	p.set_num(m_mpNum*c);
    	p.set_denom(m);

    	//normalise q
    	c=m/q.get_denom();
    	q.set_num(q.get_num()*c);
    	q.set_denom(m);
    }else
		p.set_rat(m_mpNum, m_mpDenom);

    //add
    p.set_num(p.get_num() + q.get_num());
    while( p.get_num() >= p.get_denom() )
        p.set_num(p.get_num() - p.get_denom());

    //remove factors
	mpz_gcd(c.get_mpz_t(), p.get_num().get_mpz_t(), p.get_denom().get_mpz_t() );	// c = gcd( p.num, p.denom )
    if( c !=  1 && c != 0 ){	// c != 0,1
        p.set_num(p.get_num()/c);
        p.set_denom(p.get_denom()/c);
    }

    return p;
}

rat rat::operator-(rat q) const
{
	rat p;

	mpz_class m,c;

	if(q.get_denom() != m_mpDenom){
    	mpz_lcm( m.get_mpz_t(), m_mpDenom.get_mpz_t(), q.get_denom().get_mpz_t() );	// m = lcd( this.denom, q.denom )

    	//normalise p
    	c=m/m_mpDenom;
    	p.set_num(m_mpNum*c);
    	p.set_denom(m);

    	//normalise q
    	c=m/q.get_denom();
    	q.set_num(q.get_num()*c);
    	q.set_denom(m);
    }else
		p.set_rat(m_mpNum, m_mpDenom);

    //subtract
    if( p.get_num() >= q.get_num() )
    	p.set_num(p.get_num() - q.get_num());
    else
    	p.set_num(p.get_denom()-(q.get_num() - p.get_num()));

    //remove factors
    mpz_gcd( c.get_mpz_t(), p.get_num().get_mpz_t(), p.get_denom().get_mpz_t() );	// c = gcd( p.num, p.denom )
    if( c != 1 && c != 0 ){
        p.set_num(p.get_num()/c);
        p.set_denom(p.get_denom()/c);
    }

    return p;
}

rat rat::operator*(int m) const
{
	rat r0;
	mpz_class c;

	r0.set_rat(m_mpNum, m_mpDenom);
	r0.set_num(r0.get_num() * m);
	if( r0.get_num() < m_mpNum ){
		r0.set_rat(0,0);
		return r0;
	}

	// Mod 1
	while( r0.get_num() >= r0.get_denom() )
		r0.set_num(r0.get_num() - r0.get_denom());

	// Remove Factors
	mpz_gcd( c.get_mpz_t(), r0.get_num().get_mpz_t(), r0.get_denom().get_mpz_t() );
	if( c != 1 && c != 0 ){
		r0.set_num(r0.get_num()/c);
		r0.set_denom(r0.get_denom()/c);
	}

	return r0;
}

rat rat::operator/(int d) const
{
	rat r0;
	mpz_class c;

	r0.set_rat( m_mpNum, m_mpDenom );
	r0.set_denom( r0.get_denom() * d );
	if( r0.get_denom() < m_mpDenom ){
		r0.set_rat( 0, 0 );
		return r0;
	}

	// Mod 1
	while( r0.get_num() >= r0.get_denom() )
		r0.set_num( r0.get_num() - r0.get_denom() );

	// Remove Factors
	mpz_gcd( c.get_mpz_t(), r0.get_num().get_mpz_t(), r0.get_denom().get_mpz_t() );
	if( c != 1 && c != 0 ){
		r0.set_num( r0.get_num() / c );
		r0.set_denom( r0.get_denom() / c );
	}

	return r0;
}

rat rat::operator*( rat r ) const
{
	rat o( this->get_num() * r.get_num(), this->get_denom() * r.get_denom() );
	o.simplify();

	return o;
}

bool rat::operator>( rat r0 ) const
{
	mpz_class mr0, mr2, nr0, nr2, r=0;

	mpz_lcm( r.get_mpz_t(), r0.get_denom().get_mpz_t(), m_mpDenom.get_mpz_t() );

	//find the numerators of r0 and r2 with this new denominator
	mr0 = r / r0.get_denom();
	mr2 = r / m_mpDenom;

	nr0 = mr0 * r0.get_num();
	nr2 = mr2 * m_mpNum;

	if( nr2 > nr0 )
		return true;
	else
		return false;
}

bool rat::operator<( rat r0 ) const
{
	mpz_class mr0, mr2, nr0, nr2, r=0;

	mpz_lcm( r.get_mpz_t(), r0.get_denom().get_mpz_t(), m_mpDenom.get_mpz_t() );

	//find the numerators of r0 and r2 with this new denominator
	mr0 = r / r0.get_denom();
	mr2 = r / m_mpDenom;

	nr0 = mr0 * r0.get_num();
	nr2 = mr2 * m_mpNum;

	if( nr2 < nr0 )
		return true;
	else
		return false;
}

bool rat::operator>=( rat r0 ) const
{
	mpz_class mr0, mr2, nr0, nr2, r=0;

	mpz_lcm( r.get_mpz_t(), r0.get_denom().get_mpz_t(), m_mpDenom.get_mpz_t() );

	//find the numerators of r0 and r2 with this new denominator
	mr0 = r / r0.get_denom();
	mr2 = r / m_mpDenom;

	nr0 = mr0 * r0.get_num();
	nr2 = mr2 * m_mpNum;

	if(nr2 >= nr0)
		return true;
	else
		return false;
}

bool rat::operator<=( rat r0 ) const
{
	mpz_class mr0, mr2, nr0, nr2, r = 0;

	mpz_lcm( r.get_mpz_t(), r0.get_denom().get_mpz_t(), m_mpDenom.get_mpz_t() );

	//find the numerators of r0 and r2 with this new denominator
	mr0 = r / r0.get_denom();
	mr2 = r / m_mpDenom;

	nr0 = mr0 * r0.get_num();
	nr2 = mr2 * m_mpNum;

	if(nr2 <= nr0)
		return true;
	else
		return false;
}

//------------------------------------------------------------------------------
// class leaf member functions (unordered pair of rational numbers with label)
//------------------------------------------------------------------------------

leaf::leaf()
{
	m_rA.set_rat(0,0);
	m_rB.set_rat(0,0);
}

leaf::leaf(mpz_class d)
{
	m_rA.set_rat(0,d);
	m_rB.set_rat(0,d);
}

leaf::leaf(rat r1, rat r2)
{
	m_rA = r1;
	m_rB = r2;
}

leaf::leaf(mpz_class a, mpz_class b, mpz_class c, mpz_class d)
{
	m_rA.set_rat(a,b);
	m_rB.set_rat(c,d);
}

bool leaf::operator==(leaf q) const
{
	if(m_rA == q.get_a()){
		if(m_rB == q.get_b())
			return true;
		else
			return false;
	}else if(m_rA == q.get_b()){
		if(m_rB == q.get_a())
			return true;
		else
			return false;
	}

	return false;
}

bool leaf::operator!=(leaf q) const
{
	if(m_rA == q.get_a()){
		if(m_rB == q.get_b())
			return false;
		else
			return true;
	}else if(m_rA == q.get_b()){
		if(m_rB == q.get_a())
			return false;
		else
			return true;
	}

	return true;
}

leaf leaf::operator+(rat q) const
{
  rat ra = m_rA;
  rat rb = m_rB;

  leaf l(ra + q, rb + q);

  return l;
}

leaf leaf::operator/(int q) const
{
  rat ra = m_rA;
  rat rb = m_rB;

  leaf l(ra/q, rb/q);

  return l;
}

void leaf::map_forward()
{
	m_rA = m_rA * 2;
	m_rB = m_rB * 2;
}

rat leaf::get_length() const
{
	rat r0, one(1,1);

	// Find the biggest endpoint of leaf and subtract the smaller endpoint
	if(m_rA >= m_rB)
		r0 = m_rA - m_rB;
	else
		r0 = m_rB - m_rA;

	// Choose smallest out of r0 and 1-r0
	one = one - r0;
	if(one < r0)
		r0 = one;

	return r0;
}


ostream& operator<<(ostream& os, const rat& r0)
{
	os<<r0.get_num()<<'/'<<r0.get_denom();
	return os;
}

ostream& operator<<(ostream& os, const leaf& l)
{
	os<<l.get_a().get_num()<<'/'<<l.get_a().get_denom();
	os<<'-';
	os<<l.get_b().get_num()<<'/'<<l.get_b().get_denom();
	return os;
}

ostream& operator<<(ostream& os, const tri& t)
{
	os<<t.get_a().get_num()<<'/'<<t.get_a().get_denom();
	os<<'-';
	os<<t.get_b().get_num()<<'/'<<t.get_b().get_denom();
	os<<'-';
	os<<t.get_c().get_num()<<'/'<<t.get_c().get_denom();
	return os;
}

istream& operator>>( istream& is, rat& r0 )
{
	mpz_class n, d;
	is >> n;
	is.ignore( 1 );
	is >> d;
	r0.set_rat( n, d );
	return is;
}

/* istream& operator>>( istream& is, leaf& l)
{
	os<<l.get_a().get_num()<<'/'<<l.get_a().get_denom();
	os<<'-';
	os<<l.get_b().get_num()<<'/'<<l.get_b().get_denom();
	return os;
} */


//------------------------------------------------------------------------------
// class tri member functions
//------------------------------------------------------------------------------

tri::tri()
{
	m_rA.set_rat(0,0);
	m_rB.set_rat(0,0);
	m_rC.set_rat(0,0);
}

tri::tri(mpz_class d)	// Just set the denominator, numerator 0
{
	m_rA.set_rat(0,d);
	m_rB.set_rat(0,d);
	m_rC.set_rat(0,d);
}

tri::tri(rat r1, rat r2, rat r3)
{
	m_rA = r1;
	m_rB = r2;
	m_rC = r3;
}

tri::tri(mpz_class a, mpz_class b, mpz_class c, mpz_class d, mpz_class e, mpz_class f)
{
	m_rA.set_rat(a,b);
	m_rB.set_rat(c,d);
	m_rC.set_rat(e,f);
}

void tri::set_tri(mpz_class n1, mpz_class d1, mpz_class n2, mpz_class d2, mpz_class n3, mpz_class d3)
{
	m_rA.set_num(n1);
	m_rA.set_denom(d1);
	m_rB.set_num(n2);
	m_rB.set_denom(d2);
	m_rC.set_num(n3);
	m_rC.set_denom(d3);
}

// Helper Functions
void tri::map_forward()
{
	m_rA = m_rA * 2;
	m_rB = m_rB * 2;
	m_rC = m_rC * 2;
}

rat tri::get_middle() const
{
	leaf lA( m_rB, m_rC ), lB( m_rA, m_rC ), lC( m_rA, m_rB );

	if( lA.get_length() > lB.get_length() ){
		if( lA.get_length() > lC.get_length() )
			return m_rA;
		else
			return m_rC;
	}else{
		if( lB.get_length() > lC.get_length() )
			return m_rB;
		else
			return m_rC;
	}
}

bool tri::has_vertex( rat r0 ) const
{
	if( m_rA == r0 )
		return true;
	if( m_rB == r0 )
		return true;
	if( m_rC == r0 )
		return true;
	return false;
}

bool tri::operator==( tri q ) const
{
cout<<"\nStarting triangle equality checker.";
	if( m_rA == q.get_a() ){
		if( m_rB == q.get_b() ){
			if( m_rC == q.get_c() )
				return true;
		}else if( m_rB == q.get_c() ){
			if( m_rC == q.get_b() )
				return true;
		}
	}else if( m_rA == q.get_b() ){
		if( m_rB == q.get_a() ){
			if( m_rC == q.get_c() )
				return true;
		}else if( m_rB == q.get_c() ){
			if( m_rC == q.get_a() )
				return true;
		}
	}else if( m_rA == q.get_c() ){
		if( m_rB == q.get_b() ){
			if( m_rC == q.get_a() )
				return true;
		}else if( m_rB == q.get_a() ){
			if( m_rC == q.get_b() )
				return true;
		}
	}

	return false;
}

bool tri::operator!=(tri q) const
{
	tri t( m_rA, m_rB, m_rC );
	return !(t == q);
}

tri tri::operator+( rat q ) const
{
	tri temp( m_rA, m_rB, m_rC );

	temp.set_a( temp.get_a() + q );
	temp.set_b( temp.get_b() + q );
	temp.set_c( temp.get_c() + q );

	return temp;
}

tri tri::operator-( rat q ) const
{
	tri temp( m_rA, m_rB, m_rC );

	temp.set_a( temp.get_a() - q );
	temp.set_b( temp.get_b() - q );
	temp.set_c( temp.get_c() - q );

	return temp;
}

tri tri::operator*( int q ) const
{
	tri temp( m_rA, m_rB, m_rC );

	temp.set_a( temp.get_a() * q );
	temp.set_b( temp.get_b() * q );
	temp.set_c( temp.get_c() * q );

	return temp;
}

tri tri::operator/( int q ) const
{
	tri temp( m_rA, m_rB, m_rC );

	temp.set_a( temp.get_a() / q );
	temp.set_b( temp.get_b() / q );
	temp.set_c( temp.get_c() / q );

	return temp;
}

#endif
