// classes for rational numbers and leaves

#if !defined(CLASSH)
	#define CLASSH

using namespace std;

//------------------------------------------------------------------------------
// class rat (rational number)
//------------------------------------------------------------------------------
class rat{
		// Member vars
		mpz_class m_mpNum;
		mpz_class m_mpDenom;
public:
		// Constructor Destructor
		rat();
		rat( mpz_class p, mpz_class q );
		~rat()					{};

		// Member functions
		void set_num(mpz_class n)				{m_mpNum = n; }
		void set_denom(mpz_class d)				{m_mpDenom = d; }
		void set_rat(mpz_class n, mpz_class d)	{m_mpNum = n; m_mpDenom = d; }

		mpz_class get_num()		const		{return m_mpNum; }
		mpz_class get_denom()	const		{return m_mpDenom; }

		// Helper Functions
		void simplify();
		bool operator==(rat q) const;
		bool operator!=(rat q) const;
		rat operator+(rat q) const;
		rat operator-(rat q) const;
		rat operator*(int m) const;
		rat operator*(rat r) const;
		rat operator/(int d) const;
		bool operator>(rat r0) const;
		bool operator<(rat r0) const;
		bool operator>=(rat r0) const;
		bool operator<=(rat r0) const;
};

//------------------------------------------------------------------------------
// class leaf (unordered pair of rational numbers)
//------------------------------------------------------------------------------
class leaf{
		// Member vars
		rat m_rA;
		rat m_rB;

public:
		// Constructor Destructor
		leaf();
		leaf(mpz_class d);	// Just set the denominator, numerator 0
		leaf(rat r1, rat r2);
		leaf(mpz_class a, mpz_class b, mpz_class c, mpz_class d);
		~leaf() 				{;}

		// Member functions
		void set_leaf(rat r1, rat r2)				{m_rA = r1; m_rB = r2; }
		void set_leaf(mpz_class n1, mpz_class d1, mpz_class n2, mpz_class d2)
													{m_rA.set_rat(n1,d1);
													m_rB.set_rat(n2,d2); }
		void set_a(rat r1)							{m_rA = r1; }
		void set_a(mpz_class n, mpz_class d)		{m_rA.set_rat(n,d); }
		void set_b(rat r2)							{m_rB = r2; }
		void set_b(mpz_class n, mpz_class d)		{m_rB.set_rat(n,d); }

		rat get_a()	const			{return m_rA; }
		rat get_b()	const			{return m_rB; }

		void simp_a()				{ m_rA.simplify(); }
		void simp_b()				{ m_rB.simplify(); }

		// Helper Functions
		void map_forward();
		rat get_length() const;

		bool operator==(leaf q) const;
		bool operator!=(leaf q) const;
	    leaf operator+(rat q) const;
		leaf operator/(int q) const;
};

//------------------------------------------------------------------------------
// class triangle (unordered triplet of rational numbers)
//------------------------------------------------------------------------------
class tri{
		// Member vars
		rat m_rA;
		rat m_rB;
		rat m_rC;

public:
		// Constructor Destructor
		tri();
		tri( mpz_class d );	// Just set the denominator, numerator 0
		tri( rat r1, rat r2, rat r3 );
		tri( mpz_class a, mpz_class b, mpz_class c, mpz_class d, mpz_class e, mpz_class f );
		~tri() 				{;}

		// Member functions
		void set_tri(rat r1, rat r2, rat r3, char c='?')	{m_rA = r1; m_rB = r2; m_rC = r3;}
		void set_tri(mpz_class n1, mpz_class d1, mpz_class n2, mpz_class d2, mpz_class n3, mpz_class d3);
		void set_a(rat r1)				{m_rA = r1; }
		void set_b(rat r2)				{m_rB = r2; }
		void set_c(rat r3)				{m_rC = r3; }
		void set_a(mpz_class n, mpz_class d)				{m_rA.set_num(n); m_rA.set_denom(d); }
		void set_b(mpz_class n, mpz_class d)				{m_rB.set_num(n); m_rB.set_denom(d); }
		void set_c(mpz_class n, mpz_class d)				{m_rC.set_num(n); m_rC.set_denom(d); }

		rat get_a()	const			{return m_rA; }
		rat get_b()	const			{return m_rB; }
		rat get_c()	const			{return m_rC; }

		// Helper Functions
		void map_forward();
		rat get_middle() const;
		bool has_vertex(rat r0) const;

		bool operator==(tri q) const;
		bool operator!=(tri q) const;
		tri operator+(rat q) const;
		tri operator-(rat q) const;
		tri operator*(int q) const;
		tri operator/(int q) const;
};

#endif
