#if !defined(GETRANGE)
	#define GETRANGE

#include "classes.cpp"
#include "leaf_helpers.hpp"

using namespace std;

#if !defined(SYMB_CONSTS)
	#define SYMB_CONSTS
const short R1 = 1;
const short R2 = 2;
const short R3 = 3;
const short UC = 4;
const short BC = 5;
const short L3 = 6;
const short L2 = 7;
const short L1 = 8;
const short L = 9;
const short R = 10;
const short C = 11;
#endif

void write_word( short* word, int length, bool empties = false );
void letter_out( short unsigned su, bool empties = false );
rat mod1( rat rat1 );
void rem_inv_c( string* word, bool full = false );
void rem_zeros( short* word, int length );

//------------------------------------------------------------------------------
// get_word_label(short[50]) -	takes symbolic word in labelling of L_{3,7} and
//							 	returns the label none=0, T=1, -T=2, other=3
//------------------------------------------------------------------------------

int get_word_label( short* word, int length )
{
	//char c;
    int i, j, current;
	bool shift, b_Tin[3]={ true, true, true}, b_mTin[3]={ true, true, true };
	rat T[3], mT[3];
    leaf range_l, range_h;

    // Set up T and mT
    T[0].set_rat( 3, 7 );
    T[1].set_rat( 5, 7 );
    T[2].set_rat( 6, 7 );
    mT[0].set_rat( 3, 14 );
    mT[1].set_rat( 5, 14 );
    mT[2].set_rat( 13, 14 );

	// Set all non-symbols to 0
	for( i = 0; i < length; ++i ){
		if( ( word[i] > C ) || ( word[i] < 0 ) )
			word[i] = 0;
	}

	// shift all letters to left
	for( i = 0; i < length; ++i ){						// For each letter in word
		if( word[i] == 0 ){							// if not a symbol
			shift = false;
			for( j = i + 1; j < length; ++j){			// go through every following letter
				if( word[j] != 0 ){					// and if that IS a symbol
					word[i] = word[j];				// shift it to the first space
					word[j] = 0;
					shift = true;					// note that something has been shifted
					j = length;							// break for loop
				}
			}
			if( !shift )							// if nothing was shifted
				i = length;
		}
	}

	// Get current letter (first letter in word)
	current = 0;

	while( ( word[ current ] != 0 ) && ( current < length ) ){
		// map forward if not first cycle
		if( current != 0 ){
			T[0] = T[0] * 2;
			T[1] = T[1] * 2;
			T[2] = T[2] * 2;
			mT[0] = mT[0] * 2;
			mT[1] = mT[1] * 2;
			mT[2] = mT[2] * 2;
		}

		// set up ranges according to current letter
		switch( word[ current ] ){
			case L1:
				range_l.set_a( 3, 7 );
				range_l.set_b( 4, 7 );
				range_h.set_a( 3, 7 );
				range_h.set_b( 4, 7 );
				break;
			case L2:
				range_l.set_a( 4, 7 );
				range_l.set_b( 9, 14 );
				range_h.set_a( 5, 14 );
				range_h.set_b( 3, 7 );
				break;
			case L3:
				range_l.set_a( 9, 14 );
				range_l.set_b( 5, 7 );
				range_h.set_a( 2, 7 );
				range_h.set_b( 5, 14 );
				break;
			case R1:
				range_l.set_a( 13, 14 );
				range_l.set_b(  1, 14 );
				range_h.set_a( 13, 14 );
				range_h.set_b(  1, 14 );
				break;
			case R2:
				range_l.set_a(  6,  7 );
				range_l.set_b( 13, 14 );
				range_h.set_a(  1, 14 );
				range_h.set_b(  1,  7 );
				break;
			case R3:
				range_l.set_a( 11, 14 );
				range_l.set_b(  6,  7 );
				range_h.set_a(  1,  7 );
				range_h.set_b(  3, 14 );
				break;
			case UC:
				// Drop through to C
				//word[current] = C;
			case BC:
				// Drop through to C
				//word[current] = C;
			case C:
				range_l.set_a(  5,  7 );
				range_l.set_b( 11, 14 );
				range_h.set_a(  3, 14 );
				range_h.set_b(  2,  7 );
				break;
			default:
				range_l.set_a( 0, 0 );
				range_l.set_b( 0, 0 );
				range_h.set_a( 0, 0 );
				range_h.set_b( 0, 0 );
				break;
		}

		// check which points are in ranges
		for( i = 0; i < 3; ++i ){
			if( !rat_in_CLOSED( T[i], range_l ) && !rat_in_CLOSED( T[i], range_h ) )
				b_Tin[i] = false;
			if( !rat_in_CLOSED( mT[i], range_l ) && !rat_in_CLOSED( mT[i], range_h ) )
				b_mTin[i] = false;
		}

		// if still some points left increment `current', else break loop
		if( b_Tin[0] || b_Tin[1] || b_Tin[2]
				|| b_mTin[0] || b_mTin[1] || b_mTin[2] )
			++current;
		else
			current = length;
	}

	if( ( b_Tin[0] || b_Tin[1] || b_Tin[2] ) &&
			( b_mTin[0] || b_mTin[1] || b_mTin[2] ) ){
		if( word[0] == L2 )
			return 2;
		else if( word[0] == R2 )
			return 1;
	}else if( b_mTin[0] || b_mTin[1] || b_mTin[2] )
		return 2;
	else if( b_Tin[0] || b_Tin[1] || b_Tin[2] )
		return 1;

	return 3;
}

//------------------------------------------------------------------------------
// get_aero_range( char, int, bool, bool) - takes symbolic word in labelling of
//							L_{3,7} and returns the range on S^1 labelled by it.
//							High is provided so that only one interval has to be
//							returned. high_c sets C to UC if true.
//------------------------------------------------------------------------------

leaf get_aero_range( short* word, int length, bool high = true, bool high_c = true )
{
	char c;
    int i, j, current;
    rat half( 1, 2 ), zero( 0, 1 ), l_null( 0, 0 );
    leaf range, current_letter[2];
	bool shift;

	// Set all non-symbols to 0
	for( i = 0; i < length; ++i){
		if( word[i] > C )
			word[i] = 0;
	}

	// shift all letters to left
	for( i = 0; i < length; ++i ){						// For each letter in word
		if( word[i] == 0 ){							// if not a symbol
			shift = false;
			for( j = i + 1; j < length; ++j){			// go through every following letter
				if(word[j] != 0){					// and if that IS a symbol
					word[i] = word[j];				// shift it to the first space
					word[j] = 0;
					shift = true;					// note that something has been shifted
					j = length;							// break for loop
				}
			}
			if( !shift )							// if nothing was shifted
				i = length;
		}
	}

	// Get current letter (last letter in word)
	i = 0;
	current = -1;
	while( word[i] != 0 && i < length )
		current = i++;

	// Set up range
	if( current >= 0 ){
		switch( word[current] ){
			case L1:
				range.set_a( 3, 7 );
				range.set_b( 4, 7 );
				break;
			case L2:
				if( high ){
					range.set_a( 5, 14 );
					range.set_b( 3,  7 );
				}else{
					range.set_a( 4,  7 );
					range.set_b( 9, 14 );
				}
				break;
			case L3:
				if( high ){
					range.set_a( 2,  7 );
					range.set_b( 5, 14 );
				}else{
					range.set_a( 9, 14 );
					range.set_b( 5,  7 );
				}
				break;
			case R1:
				range.set_a( 13, 14 );
				range.set_b(  1, 14 );
				break;
			case R2:
				if( high ){
					range.set_a( 1, 14 );
					range.set_b( 1, 7 );
				}else{
					range.set_a(  6,  7 );
					range.set_b( 13, 14 );
				}
				break;
			case R3:
				if( high ){
					range.set_a( 1,  7 );
					range.set_b( 3, 14 );
				}else{
					range.set_a( 11, 14 );
					range.set_b(  6,  7 );
				}
				break;
			case C:
				if( high ){
					range.set_a( 3, 14 );
					range.set_b( 2,  7 );
				}else{
					range.set_a(  5,  7 );
					range.set_b( 11, 14 );
				}
				break;
			case UC:
				range.set_a( 3, 14 );
				range.set_b( 2,  7 );
				break;
			case BC:
				range.set_a(  5,  7 );
				range.set_b( 11, 14 );
				break;
			default:
				range.set_a( 0, 0 );
				range.set_b( 0, 0 );
				break;
		}


		while( --current >= 0 ){
			// Find one pre-image of range
			if( rat_in(zero,range) ){	// if range spans zero
				range.set_a( range.get_a() / 2 + half );
				range.set_b( range.get_b() / 2 );
			}else{
				range.set_a( range.get_a() / 2 );
				range.set_b( range.get_b() / 2 );
			}

			// Set current letter
			switch( word[current] ){
				case L1:
					current_letter[0].set_a( 3, 7 );
					current_letter[0].set_b( 4, 7 );
					current_letter[1].set_a( 3, 4 );
					current_letter[1].set_b( 4, 7 );
					break;
				case L2:
					current_letter[0].set_a( 5, 14 );
					current_letter[0].set_b( 3,  7 );
					current_letter[1].set_a( 4,  7 );
					current_letter[1].set_b( 9, 14 );
					break;
				case L3:
					current_letter[0].set_a( 2,  7 );
					current_letter[0].set_b( 5, 14 );
					current_letter[1].set_a( 9, 14 );
					current_letter[1].set_b( 5,  7 );
					break;
				case R1:
					current_letter[0].set_a(  0,  1 );
					current_letter[0].set_b(  1, 14 );
					current_letter[1].set_a( 13, 14 );
					current_letter[1].set_b(  1,  1 );
					break;
				case R2:
					current_letter[0].set_a(  1, 14 );
					current_letter[0].set_b(  1,  7 );
					current_letter[1].set_a(  6,  7 );
					current_letter[1].set_b( 13, 14 );
					break;
				case R3:
					current_letter[0].set_a(  1,  7 );
					current_letter[0].set_b(  3, 14 );
					current_letter[1].set_a( 11, 14 );
					current_letter[1].set_b(  6,  7 );
					break;
				case C:
					current_letter[0].set_a(  3, 14 );
					current_letter[0].set_b(  2,  7 );
					current_letter[1].set_a(  5,  7 );
					current_letter[1].set_b( 11, 14 );
					break;
				case UC:
					current_letter[0].set_a( 3, 14 );
					current_letter[0].set_b( 2,  7 );
					current_letter[1].set_a( 3, 14 );
					current_letter[1].set_b( 2,  7 );
					break;
				case BC:
					current_letter[0].set_a(  5,  7 );
					current_letter[0].set_b( 11, 14 );
					current_letter[1].set_a(  5,  7 );
					current_letter[1].set_b( 11, 14 );
					break;
			}

			if( ( word[current] == L1 ) || ( word[current] == UC ) || ( word[current] == BC ) ){
				// Simple, one range case
				// Check if range in current_letter[0]
				if( !rat_in_CLOSED( range.get_a(), current_letter[0] ) && !rat_in_CLOSED( range.get_b(), current_letter[0] ) ){
					range = range + half;
					// Check if range+half in current_letter[0]
					if( !rat_in_CLOSED( range.get_a(), current_letter[0] ) && !rat_in_CLOSED( range.get_b(), current_letter[0] ) ){
						cout<<"\nget_aero_range: word proffered is invalid(1): ";
						write_word( word, length );
						cout<<endl;
						range.set_leaf( l_null, l_null );
					}
				}
			}else if( word[current] == R1 ){
				// One range, spanning zero (mod 1)
				// Check if range in current_letter[0]
				if( !( ( range.get_a() <= current_letter[0].get_b() ) || (range.get_a() >= current_letter[1].get_a() ) )
						&& !( ( range.get_b() <= current_letter[0].get_b() ) || ( range.get_b() >= current_letter[1].get_a() ) ) )
					range = range + half;
					// Check if range+half in current_letter[0]
					if( !( ( range.get_a() <= current_letter[0].get_b() ) || ( range.get_a() >= current_letter[1].get_a() ) )
							&& !( ( range.get_b() <= current_letter[0].get_b() ) || ( range.get_b() >= current_letter[1].get_a() ) ) ){
						cout<<"\nget_aero_range: word proffered is invalid(2).\n";
						write_word( word, length );
						range.set_leaf( l_null, l_null );
					}
			}else if( word[current] == C ){
				if( !rat_in_CLOSED( range.get_a(), current_letter[0] ) &&
						!rat_in_CLOSED( range.get_b(), current_letter[0] ) ){
					cout<<"\nget_aero_range: word proffered is invalid(3).\n";
					write_word( word, length );
					cout<<endl;
					range.set_leaf( l_null, l_null );
				}
				if( !high_c )
					range = range + half;
			}else{
				// check for BOTH ranges associated with L2,L3,R2,R3
				// Check if range in current_letter
				if( !rat_in_CLOSED( range.get_a(), current_letter[0] ) &&
						!rat_in_CLOSED( range.get_b(), current_letter[0] ) &&
						!rat_in_CLOSED( range.get_a(), current_letter[1] ) &&
						!rat_in_CLOSED( range.get_b(), current_letter[1] ) ){
					range = range + half;
					// Check if range+half in current_letter
					if( !rat_in_CLOSED( range.get_a(), current_letter[0] ) &&
							!rat_in_CLOSED( range.get_b(), current_letter[0] ) &&
							!rat_in_CLOSED( range.get_a(), current_letter[1] ) &&
							!rat_in_CLOSED( range.get_b(), current_letter[1] ) ){
						cout<<"\nget_aero_range: word proffered is invalid(4).\n";
						write_word( word, length );
						cout<<endl;
						range.set_leaf( l_null, l_null );
					}
				}
			}
		}
	}

	return range;
}

//------------------------------------------------------------------------------
// get_rabbit_range(char) - takes full word in labelling of L_{1,7} and returns
//			  				the range on S^1 labelled by it
//------------------------------------------------------------------------------

leaf get_rabbit_range( short* word, int length )
{
	int current;
	rat half( 1, 2 );
	leaf range, current_letter;

	// get rid of zeros in word
	rem_zeros( word, length );

	// Get current letter (last letter in word)
	current = length - 1;
	while( word[current] == 0 && current > -1 )
		current--;

	if( current >= 0 ){
		// Set up range
		switch( word[current] ){
			case L1:
				range.set_a( 1, 7 );
				range.set_b( 2, 7 );
				break;
			case L2:
				range.set_a( 2, 7 );
				range.set_b( 4, 7 );
				break;
			case R1:
				range.set_a(  9, 14 );
				range.set_b( 11, 14 );
				break;
			case R2:
				range.set_a( 11, 14 );
				range.set_b(  1, 14 );
				break;
			case UC:
				range.set_a( 1, 14 );
				range.set_b( 1,  7 );
				break;
			case BC:
				range.set_a( 4,  7 );
				range.set_b( 9, 14 );
				break;
			case L:
				range.set_a( 1, 7 );
				range.set_b( 4, 7 );
				break;
			case R:
				range.set_a( 9, 14 );
				range.set_b( 1, 14 );
				break;
		}

		while( --current >= 0 ){
			// Find one pre-image of range
			range.set_a( range.get_a() / 2 );
			range.set_b( range.get_b() / 2 );
			if( range.get_a() > range.get_b() ) // adjust if range spans zero
				range.set_a( mod1( range.get_a() + half ) );

			// Set current letter
			switch( word[current] ){
				case L1:
					current_letter.set_a( 1, 7 );
					current_letter.set_b( 2, 7 );
					break;
				case L2:
					current_letter.set_a( 2, 7 );
					current_letter.set_b( 4, 7 );
					break;
				case R1:
					current_letter.set_a(  9, 14 );
					current_letter.set_b( 11, 14 );
					break;
				case R2:
					current_letter.set_a( 11, 14 );
					current_letter.set_b(  1, 14 );
					break;
				case UC:
					current_letter.set_a( 1, 14 );
					current_letter.set_b( 1,  7 );
					break;
				case BC:
					current_letter.set_a( 4,  7 );
					current_letter.set_b( 9, 14 );
					break;
				case L:
					current_letter.set_a( 1, 7 );
					current_letter.set_b( 4, 7 );
					break;
				case R:
					current_letter.set_a( 9, 14 );
					current_letter.set_b( 1, 14 );
					break;
			}

			// Check if range is in current letter and adjust if not
			if( rat_in_CLOSED( range.get_a(), current_letter ) ){
				if( !rat_in_CLOSED( range.get_b(), current_letter ) )		// a in, b not
					range.set_b( current_letter.get_b() );					//
			}else if( rat_in_CLOSED( range.get_b(), current_letter ) )			//
				range.set_a( current_letter.get_a() );						// b in, a not
			else{
				range.set_a( mod1( range.get_a() + half ) );
				range.set_b( mod1( range.get_b() + half ) );
				if( rat_in_CLOSED( range.get_a(), current_letter ) ){
					if( !rat_in_CLOSED( range.get_b(), current_letter ) )	// a in, b not
						range.set_b( current_letter.get_b() );		//
				}else								//
					range.set_a( current_letter.get_a() );			// b in, a not
			}
		}
	}

	return range;
}

//------------------------------------------------------------------------------
// translate_v1( string vague, short *full) - translates a vague word from
//						L_{1/7} to a full word of short
//						integers. The short pointer full
//						dictates where the new full word
//						will be stored and should be a
//						pointer to an array of
//						short integers. The length of this array is returned
//------------------------------------------------------------------------------

int trans_v1( string vague, short *full )
{
	int i, length = 0;

	// get rid of invalid characters in vague
	rem_inv_c( &vague );

	for( i = 0; i < vague.length(); ++i){	// translate word
		switch( vague.at(i) ){
			case 'L':
				full[ length++ ] = L;
				if( i < vague.length() - 1 ){
					if( vague.at( i + 1 ) != '?' ){
						if( vague.at( i + 1 ) == 'L' )
							full[ length - 1 ] = L1;
						else
							full[ length - 1 ] = L2;
					}else{
						if( i > 0 ){
							if( vague.at( i - 1 ) == 'C' )
								full[ length - 1 ] = L1;
							else
								full[ length - 1 ] = L2;
						}
					}
				}else if( vague.at( i - 1 ) == 'C' ){
					full[ length - 1 ] = L1;
				}else{
					full[ length - 1 ] = L2;
				}
				break;
			case 'R':
				full[ length++ ] = R;
				if( i < vague.length() - 1 ){
					if( vague.at( i + 1 ) != '?' ){
						if( vague.at( i + 1 ) == 'L' )
							full[ length - 1 ] = R1;
						else
							full[ length - 1 ] = R2;
					}
				}
				break;
			case 'U':
				full[ length++ ] = UC;
				++i;
				break;
			case 'B':
				full[ length++ ] = BC;
				++i;
				break;
			case 'C':
				full[ length++ ] = C;
				break;
			default:
				break;
		}
	}

	return length;
}

//------------------------------------------------------------------------------
// trans_f1(string c_full, short *full) - Translates a full word, in the
//					    labelling of L_{1/7}, from char to
//					    short integers. (size of full must allow at least
//						for c_full.length() chars
//------------------------------------------------------------------------------

int trans_f1( string c_full, short *full )
{
	int i, length = 0;

	// get rid of invalid characters in full_c
	rem_inv_c( &c_full, true );

	for( i = 0; i < c_full.length(); ++i){
		switch( c_full.at(i) ){
			case 'L':
				if( i < c_full.length() - 1 ){
					if( c_full.at( i + 1 ) == '1' ){
						full[ length++ ] = L1;
						++i;
					}else if( c_full.at( i + 1 ) == '2' ){
						full[ length++ ] = L2;
						++i;
					}else{
						cout<<"Invalid word. (L at position "<<i<<")\n";
						return 0;
					}
				}
				break;
			case 'R':
				if( i < c_full.length() - 1 ){
					if( c_full.at( i + 1 ) == '1' ){
						full[ length++ ] = R1;
						++i;
					}else if( c_full.at( i + 1 ) == '2' ){
						full[ length++ ] = R2;
						++i;
					}else{
						cout<<"Invalid word. (R at position "<<i<<")\n";
						return 0;
					}
				}
				break;
			case 'U':
				full[ length++ ] = UC;
				if( c_full.at( i + 1 ) == 'C' )
					++i;
				break;
			case 'B':
				full[ length++ ] = BC;
				if( c_full.at( i + 1 ) == 'C' )
					++i;
				break;
			case 'C':
				full[ length++ ] = C;
				break;
			default:
				break;
		}
	}

	// get rid of zeros in full
	// rem_zeros( full, length );

	return length;
}

//------------------------------------------------------------------------------
// chk_word1( short* word, int length) - 	Checks that word is a valid
//								word in L_{1/7}. Will not work if irregular
//								zeros in word. Returns place of first
//								error, or parameter length if no error
//								found.
//------------------------------------------------------------------------------

int chk_word1( const short* const word, const int length )
{
	int i;

	for ( i = 1; i < length; ++i){
		switch( word[i] ){
			case L:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L1 ) &&
						( word[ i - 1 ] != UC ) && ( word[ i - 1 ] != C ) &&
						( word[ i - 1 ] != BC ) && ( word[ i - 1 ] != R1 ) )
					return i;
				break;
			case R:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R2 ))
					return i;
				break;
			case L1:
				if( ( word[ i - 1 ] != UC ) && ( word[ i - 1 ] != C ) && ( word[ i - 1 ] != BC ) )
					return i;
				break;
			case L2:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L1 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R1 ) )
					return i;
				break;
			case R1:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R2 ) )
					return i;
				break;
			case R2:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R2 ) )
					return i;
				break;
			case C:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R2 ) )
					return i;
				break;
			case UC:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R2 ) )
					return i;
				break;
			case BC:
				if( ( word[ i - 1 ] != L ) && ( word[ i - 1 ] != L2 ) && ( word[ i - 1 ] != R ) && ( word[ i - 1 ] != R2 ) )
					return i;
				break;
			default:
				i = length;
				break;
		}
	}

	return length;
}

//------------------------------------------------------------------------------
// Little helper functions
//------------------------------------------------------------------------------

rat mod1( rat rat1 )
{
	while( rat1.get_num() >= rat1.get_denom() )
		rat1.set_num( rat1.get_num() - rat1.get_denom() );

	return rat1;
}

//------------------------------------------------------------------------------
// Shorthand functions
//------------------------------------------------------------------------------

void write_word( short* word, int length, bool empties )
{
	int i;

	cout << endl;
	for( i = 0; i < length; ++i)
		letter_out( word[i], empties );

	cout << endl;
}

void letter_out( short unsigned su, bool empties )
{
	switch( su ){
		case L1:
			cout<<"L1";
			break;
		case L2:
			cout<<"L2";
			break;
		case L3:
			cout<<"L3";
			break;
		case UC:
			cout<<"UC";
			break;
		case BC:
			cout<<"BC";
			break;
		case R3:
			cout<<"R3";
			break;
		case R2:
			cout<<"R2";
			break;
		case R1:
			cout<<"R1";
			break;
		case R:
			cout<<"R";
			break;
		case L:
			cout<<"L";
			break;
		case C:
			cout<<"C";
			break;
		default:
			if( empties )
				cout<<'0';
			break;
	}
}

//------------------------------------------------------------------------------
//	rem_inv_c( string* word, bool full ) - Removes the invalid chars from word.
//						full signifies whether or not the word is full, rather
//						than vague.
//------------------------------------------------------------------------------

void rem_inv_c( string* word, bool full )
{
	int i = 0;
	string buffer;

	for( ; i < word->length(); ++i ){
		if( word->at(i) == 'U' || word->at(i) == 'B' || word->at(i) == 'C'){
			buffer += word->at(i);
		}else if( word->at(i) == 'L' || word->at(i) == 'R' ){
			if( full ){
				if( ( word->at( i + 1 ) == '1' || word->at( i + 1 ) == '2' ) && ( i + 1 ) < word->length() ){
					buffer += word->at(i);
					buffer += word->at( ++i );
				}
			}else
				buffer += word->at(i);
		}
	}

	word->clear();
	word->replace( 0, word->length(), buffer );
}

//------------------------------------------------------------------------------
//	rem_zeros(short *word) - Pushes teh zeros to the end of word.
//------------------------------------------------------------------------------

void rem_zeros( short *word, int length )
{
	int i, j;
	bool shift;

	for( i = 0; i < length; ++i ){						// For each letter in word
		shift = true;
		while( ( word[i] == 0 ) && ( shift == true ) ){	// while zero and something was shifted last time
			shift = false;
			for( j = i + 1; j < length; ++j ){			// go through every following letter
				if(word[j] != 0){						// and if that's non-zero
					word[ j - 1 ] = word[j];				// shift it forward one
					word[j] = 0;
					shift = true;						// note that something has been shifted
				}
			}
		}
	}
}
#endif
