#ifndef LHELP
	#define LHELP

#include "classes.cpp"

using namespace std;

//------------------------------------------------------------------------------
// point_bounded(rat p, leaf l)
//------------------------------------------------------------------------------

bool point_bounded(rat p, leaf l){

	rat half(1,2), temp;
	bool b = false;

	if(l.get_a() > l.get_b()){	// if a > b
		temp = l.get_a();
		l.set_a(l.get_b());
		l.set_b(temp);
	}

	if((p > l.get_a()) && (l.get_b() > p))
		b = true;

	l.set_a(l.get_a() + half);
	if(l.get_b() > l.get_a())
		b = !b;

	if((p == l.get_a()) || (p == l.get_b()))
		b = false;

	return b;
}

//------------------------------------------------------------------------------
// leaf_cross(leaf,leaf) - returns TRUE if the leaves cross, FALSE otherwise
//------------------------------------------------------------------------------

bool leaf_cross(leaf one, leaf two)
{
	// Check for end of file leaves
	if(one.get_a().get_denom()== 0 || one.get_b().get_denom() == 0 || 
			two.get_a().get_denom() == 0 || two.get_b().get_denom() == 0)
		return false;

	// Check endpoints aren't common between one and two, and for a single shared endpoint
	if((one.get_a() == two.get_a()) || (one.get_a() == two.get_b()))
		return false;
	if((one.get_b() == two.get_a()) || (one.get_b() == two.get_b()))
		return false;

	// Leaf two must have either none or both endpoints in the region bounded
	// above and below by the endpoints of leaf one

	rat one_sm, one_lg;
	bool cross = false;

	// Set one_sm and one_lg
	if(one.get_a() < one.get_b()){
		one_sm = one.get_a();
		one_lg = one.get_b();
	}else{
		one_sm = one.get_b();
		one_lg = one.get_a();
	}

	// is two.a in between one_sm and one_lg?
	if((one_sm < two.get_a()) && (one_lg > two.get_a()))
		cross = true;

	// is two.b in between one_sm and one_lg?
	if((one_sm < two.get_b()) && (one_lg > two.get_b()))
		cross = !cross;

	return cross;
}

//------------------------------------------------------------------------------
// get_preimage(leaf l, leaf major) -
// 		returns one pre-image of l in lamination with major leaf major.
//------------------------------------------------------------------------------

leaf get_preimage(leaf l, leaf major)
{
	leaf pi, major2;
	rat half(1,2);

	// calc the 2nd major leaf
	major2.set_leaf( major.get_a() + half, major.get_b() + half );

	pi.set_leaf( l.get_a() / 2, l.get_b() / 2 );

	// use long pre-image if pi crosses major, if pi longer than major, or if pi shares both endpoints with the major leaves
	if( leaf_cross(pi, major) || (pi.get_length() > major.get_length()) ||
			( ( pi.get_a() == major.get_a() ) && ( pi.get_b() == major2.get_b() ) ) ||
			( ( pi.get_a() == major.get_b() ) && ( pi.get_b() == major2.get_a() ) ) ||
			( ( pi.get_b() == major.get_a() ) && ( pi.get_a() == major2.get_b() ) ) ||
			( ( pi.get_b() == major.get_b() ) && ( pi.get_a() == major2.get_a() ) ) )
		pi.set_b( pi.get_b() + half );

	return pi;
}

//------------------------------------------------------------------------------
// Misc helper functions
//------------------------------------------------------------------------------

// set order to be big if you're looking at big numbers
rat dist(rat A, rat B)
{
	rat r0, one(1,1);

	// Find the largest point and subtract the smaller point
	if(A >= B)
		r0 = A - B;
	else
		r0 = B - A;

	// Choose smallest out of r0 and 1-r0
	one = one - r0;
	if(one < r0)
		r0 = one;

	return r0;
}

rat dist(rat a, leaf l)
{
	rat r0, r1;

	r0 = dist(a, l.get_a());
	r1 = dist(a, l.get_b());

	if(r0 < r1)
		return r0;
	else
		return r1;
}

rat dist(rat A, tri T)
{
	rat r0, r1, r2;

	r0 = dist(A, T.get_a());
	r1 = dist(A, T.get_b());
	r2 = dist(A, T.get_c());

	if(r0 < r1){
		if(r0 < r2)
			return r0;
		else
			return r2;
	}else{
		if(r1 < r2)
			return r1;
		else
			return r2;
	}
}

/*rat dist(leaf l1, leaf l2)
{
	rat r0, r1, len1, len2;
	leaf l;

	r0 = dist(l1.get_a(), l2);
	r1 = dist(l1.get_b(), l2);

	if(r0 <= r1){	// a is closer
		len1 = dist(l1.get_a(), l2.get_a());
		len2 = dist(l1.get_a(), l2.get_b());
		if(len1 <= len2){	// a is closest to a
			len2 = dist(l1.get_b(), l2.get_b());
			l.set_leaf(l1.get_b(),l2.get_b());
			if(point_bounded(l2.get_a(),l)){
				r0.set_rat(1,1);
				len2 = len2 - r0;
			}
		}else{	// a is closest to b
			len1 = dist(l1.get_b(), l2.get_a());
			l.set_leaf(l1.get_b(),l2.get_a());
			if(point_bounded(l2.get_b(),l)){
				r0.set_rat(1,1);
				len1 = len1 - r0;
			}
		}
	}else{	// b is closer
		len1 = dist(l1.get_b(), l2.get_a());
		len2 = dist(l1.get_b(), l2.get_b());
		if(len1 <= len2){	// b is closest to a
			len2 = dist(l1.get_a(), l2.get_b());
			l.set_leaf(l1.get_a(),l2.get_b());
			if(point_bounded(l2.get_a(),l)){
				r0.set_rat(1,1);
				len2 = len2 - r0;
			}
		}else{	// b is closest to b
			len1 = dist(l1.get_a(), l2.get_a());
			l.set_leaf(l1.get_a(),l2.get_a());
			if(point_bounded(l2.get_b(),l)){
				r0.set_rat(1,1);
				len1 = len1 - r0;
			}
		}
	}

	return (len1 + len2);
}*/

rat dist(leaf l1, leaf l2)
{
	rat one(1,1), len1, len2;
	leaf l;

	len1 = dist(l1.get_a(), l2);
	len2 = dist(l1.get_b(), l2);

	if(len1 <= len2){	// l1.a is closer to l2
		len2 = dist(l1.get_a(), l2.get_b());
		if(len1 != len2){	// l1.a is closest to l2.a
			len2 = dist(l1.get_b(), l2.get_b());
		}else{	// l1.a is closest to l2.b
			len1 = dist(l1.get_b(), l2.get_a());
		}
	}else{	// l1.b is closer to l2
		len1 = dist(l1.get_b(), l2.get_a());
		if(len1 != len2){	// b is closest to b
			len1 = dist(l1.get_a(), l2.get_a());
		}else{	// b is closest to a
			len1 = dist(l1.get_a(), l2.get_b());
		}
	}

	return (len1 + len2);
}

rat dist(leaf l, tri T)
{
	rat r0, r1;

	r0 = dist(l.get_a(), T);
	r1 = dist(l.get_b(), T);

	if(r0 < r1)
		return r0;
	else
		return r1;
}

bool rat_in( rat r0, rat a, rat b )	// determins whether r0 lies inside (a,b) (clockwise from a to b)
{
	// mod1 all rats
	while( r0.get_num() >= r0.get_denom() )
		r0.set_num( r0.get_num() - r0.get_denom() );
	while( a.get_num() >= a.get_denom() )
		a.set_num( a.get_num() - a.get_denom() );
	while( b.get_num() >= b.get_denom() )
		b.set_num( b.get_num() - b.get_denom() );

	if(a > b){	// if (a,b) spans zero
		if((r0 < b) || (r0 > a))
			return true;
		else
			return false;
	}else{
		if((r0 < b) && (r0 > a))
			return true;
		else
			return false;
	}
}

bool rat_in(rat r0, mpz_class n1, mpz_class d1, mpz_class n2, mpz_class d2)	// determins whether r0 lies inside (a,b), assumes a < b
{
	rat a( n1, d1 ), b( n2, d2 );

	return rat_in( r0, a, b );
}

bool rat_in(rat r0, leaf l)	// determins whether r0 lies inside (l.a,l.b) (clockwise from a to b)
{
	return rat_in(r0,l.get_a(),l.get_b());
}

bool rat_in_CLOSED(rat r0, rat a, rat b)	// determins whether r0 lies inside (a,b) (clockwise from a to b)
{
	// mod1 all rats
	while(r0.get_num() >= r0.get_denom())
		r0.set_num(r0.get_num() - r0.get_denom());
	while(a.get_num() >= a.get_denom())
		a.set_num(a.get_num() - a.get_denom());
	while(b.get_num() >= b.get_denom())
		b.set_num(b.get_num() - b.get_denom());

	if(a > b){	// if (a,b) spans zero
		if((r0 <= b) || (r0 >= a))
			return true;
		else
			return false;
	}else{
		if((r0 <= b) && (r0 >= a))
			return true;
		else
			return false;
	}
}

bool rat_in_CLOSED(rat r0, leaf l)	// determins whether r0 lies inside (a,b) (clockwise from a to b)
{
	return rat_in_CLOSED(r0,l.get_a(),l.get_b());
}

bool rat_in_CLOSED(rat r0, mpz_class n1, mpz_class d1, mpz_class n2, mpz_class d2)	// determins whether r0 lies inside (a,b), assumes a < b
{
	rat a(n1,d1), b(n2,d2);

	return rat_in_CLOSED(r0,a,b);
}

//------------------------------------------------------------------------------
// leaf find_approxer(leaf approxee, leaf major, leaf l1, leaf l2) - Calculates
//		the which of l1 or l2 is the best approximation of approxee, assuming
//		that l1 and l2 are diametrically opposite (in this case the two 
//		pre-images of a leaf). Major must be the major leaf of the lamination. 
//			Returns the best approximator.
//------------------------------------------------------------------------------

leaf find_approxer(leaf approxee, leaf major, leaf l1, leaf l2){

	bool b1,b2,ba,reverse = false;
	rat half(1,2), one(1,1);
	leaf ctop;
	
	ctop.set_a(major.get_a());
	ctop.set_b(major.get_b() + half);
	while(ctop.get_b().get_num() > ctop.get_b().get_denom())
		ctop.set_b(ctop.get_b().get_num() - ctop.get_b().get_denom(), ctop.get_b().get_denom());

	if(approxee == major){
		approxee.set_a(one - approxee.get_a());
		approxee.set_b(one - approxee.get_b());
		reverse = true;
	}

	b1 = point_bounded(l1.get_a(), major);
	b2 = point_bounded(l2.get_a(), major);
	ba = point_bounded(approxee.get_a(), major);
	
	if(b1 == b2){
		b1 = point_bounded(l1.get_a(), ctop);
		b2 = point_bounded(l2.get_a(), ctop);
		ba = point_bounded(approxee.get_a(), ctop);
		
		if(b1 == ba){
			if(!reverse)
				return l1;
			else
				return l2;
		}else{
			if(!reverse)
				return l2;
			else
				return l1;
		}
	}else{
		if(b1 == ba){
			if(!reverse)
				return l1;
			else
				return l2;
		}else{
			if(!reverse)
				return l2;
			else
				return l1;
		}
	}
}

#endif
